import { QuickJSContext, QuickJSHandle, QuickJSRuntime, VmCallResult } from "quickjs-emscripten";
import { JSFunction, JSFunctionAsync, JSValue } from "@canvas-js/utils";
import { VMError } from "./error.js";
export interface VMOptions {
    log?: (...args: JSValue[]) => void;
    runtimeMemoryLimit?: number;
}
export declare class VM {
    #private;
    readonly runtime: QuickJSRuntime;
    readonly context: QuickJSContext;
    static RUNTIME_MEMORY_LIMIT: number;
    private readonly log;
    static initialize(options?: VMOptions): Promise<VM>;
    private constructor();
    /**
     * Cleans up this VM instance.
     */
    dispose(): void;
    setGlobalValues(values: Record<string, QuickJSHandle>): void;
    execute(contract: string, options?: {
        uri?: string;
    }): void;
    import(contract: string, options?: {
        uri?: string;
    }): QuickJSHandle;
    get: (path: string) => QuickJSHandle;
    /**
     * Resolving promises inside QuickJS is tricky because you have to call
     * runtime.executePendingJobs() to get the promise to resolve, so you
     * can't use await syntax even though context.resolvePromise returns a
     * native Promise. This is a utility method that lets you use await.
     */
    resolvePromise: (promise: QuickJSHandle) => Promise<QuickJSHandle>;
    call: (fn: string | QuickJSHandle, thisArg: string | QuickJSHandle, args: QuickJSHandle[]) => QuickJSHandle;
    callAsync: (fn: string | QuickJSHandle, thisArg: string | QuickJSHandle, args: QuickJSHandle[]) => Promise<QuickJSHandle>;
    unwrapError: (handle: QuickJSHandle) => VMError;
    wrapError: (err: any) => QuickJSHandle;
    unwrapResult: (result: VmCallResult<QuickJSHandle>) => QuickJSHandle;
    getBoolean: (handle: QuickJSHandle) => boolean;
    getUint8Array: (handle: QuickJSHandle) => Uint8Array;
    newUint8Array: (value: Uint8Array) => QuickJSHandle;
    /**
     * Wrap an array outside a QuickJS VM by one level,
     * returning a QuickJSHandle in the host environment.
     * `wrapArray` disposes all of its handle elements.
     */
    wrapArray: (array: QuickJSHandle[]) => QuickJSHandle;
    /**
     * Unwrap an array inside a QuickJS VM by one level,
     * returning a QuickJSHandle[] in the host environment.
     * `unwrapArray` does NOT dispose of the original handle.
     */
    unwrapArray: <T = QuickJSHandle>(handle: QuickJSHandle, map?: (elementHandle: QuickJSHandle) => T) => T[];
    /**
     * Wrap an object outside a QuickJS VM by one level,
     * returning a QuickJSHandle in the host environment.
     * `wrapObject` disposes all of its handle values.
     */
    wrapObject: (object: Record<string, QuickJSHandle>) => QuickJSHandle;
    /**
     * Unwrap an object inside a QuickJS VM by one level,
     * returning a Record<string, QuickJSHandle> in the host environment.
     * `unwrapObject` does NOT dispose of the original handle.
     */
    unwrapObject: <T = QuickJSHandle>(handle: QuickJSHandle, map?: (propertyHandle: QuickJSHandle) => T) => Record<string, T>;
    wrapValue: (value: JSValue) => QuickJSHandle;
    is: (a: QuickJSHandle, b: QuickJSHandle) => boolean;
    isArray: (handle: QuickJSHandle) => boolean;
    isInstanceOf: (instanceHandle: QuickJSHandle, classHandle: QuickJSHandle) => boolean;
    isUint8Array: (handle: QuickJSHandle) => boolean;
    unwrapValue: (handle: QuickJSHandle) => JSValue;
    wrapFunction: (fn: JSFunction | JSFunctionAsync) => QuickJSHandle;
    unwrapFunction: (handle: QuickJSHandle, thisArg?: QuickJSHandle) => JSFunction;
    unwrapFunctionAsync: (handle: QuickJSHandle, thisArg?: QuickJSHandle) => JSFunctionAsync;
    cache: (handle: QuickJSHandle) => QuickJSHandle;
}
